@@ -1,5 +1,7 @@
-from tethys_wrf.main import *
+from tethys_wrf.main import WRF
+# from tethys_wrf import sio
 from functools import wraps
+import pyproj
 
 # Default proj
 wgs84 = pyproj.Proj(proj='latlong', datum='WGS84')
