@@ -12,12 +12,14 @@ import numpy as np
 # from netCDF4 import Dataset
 # from wrf import getvar, interpline, CoordPair, xy_to_ll, ll_to_xy
 # import wrf
+from tethys_wrf import virtual_parameters as vp
+# import virtual_parameters as vp
 import copy
-import orjson
+# import orjson
 import tethys_utils as tu
 from tethys_wrf import sio
-from multiprocessing.pool import ThreadPool, Pool
-import zstandard as zstd
+# from multiprocessing.pool import ThreadPool, Pool
+# import zstandard as zstd
 
 ##############################################
 ### Parameters
@@ -79,6 +81,7 @@ class WRF(object):
         setattr(self, 'data', xr1)
         setattr(self, 'mappings', wrf_mapping)
         setattr(self, 'datasets', dsb)
+        setattr(self, 'vp', vp)
 
         pass
 
@@ -87,140 +90,6 @@ class WRF(object):
         return repr(self.data)
 
 
-    @staticmethod
-    def calc_rh2(wrf_xr, method=1):
-        """
-        Two methods to calc relative humidity at 2 meters. Copied from https://github.com/keenmisty/WRF/blob/master/matlab_scripts/functions/CalRH.m.
-        I do not know where the first method comes from, but the second method is the algorithm used in ncl (wrf_rh).
-
-        Parameters
-        ----------
-        wrf_xr : xr.Dataset
-            The complete WRF output dataset with Q2, T2, and PSFC.
-        method : int
-            The method to calc RH.
-
-        Returns
-        -------
-        xr.DataArray
-        """
-        ## Assign variables
-        qv = wrf_xr['Q2']
-        t = wrf_xr['T2']
-        pres = wrf_xr['PSFC']
-
-        if method == 1:
-            e = pres*qv/(0.622+qv)
-            E = 610.78*10**(7.5*(t-273.15)/(t-36.16))
-            rh = e/E*100
-        elif method == 2:
-            es = 6.112*np.exp(17.67*(t-273.15)/(t-29.65))
-            qvs = 0.622*es/(pres/100-(1-0.622)*es)
-            rh = 100*qv/qvs
-        else:
-            raise ValueError('method must be either 1 or 2')
-
-        rh = xr.where(rh > 100, 100, rh)
-
-        return rh
-
-
-    @staticmethod
-    def calc_wind_speed(wrf_xr, height=2):
-        """
-        Estimate the mean wind speed at 10 or 2 m from the V and U WRF vectors of wind speed. The 2 m method is according to the FAO 56 paper.
-
-        Parameters
-        ----------
-        wrf_xr : xr.Dataset
-            The complete WRF output dataset with Q2, T2, and PSFC.
-        height : int
-            The height for the estimate.
-
-        Returns
-        -------
-        xr.DataArray
-        """
-        u10 = wrf_xr['U10']
-        v10 = wrf_xr['V10']
-
-        ws = np.sqrt(u10**2 + v10**2)
-
-        if height == 2:
-            ws = ws*4.87/(np.log(67.8*10 - 5.42))
-        elif height != 10:
-            raise ValueError('height must be either 10 or 2.')
-
-        return ws
-
-
-    @staticmethod
-    def calc_temp2(wrf_xr, units='degC'):
-        """
-
-        """
-        t2 = wrf_xr['T2']
-
-        if units == 'degC':
-            t2 = t2 - 273.15
-        elif units != 'K':
-            raise ValueError('units must be either degC or K.')
-
-        return t2
-
-
-    @staticmethod
-    def calc_surface_pressure(wrf_xr, units='hPa'):
-        """
-
-        """
-        pres = wrf_xr['PSFC']
-
-        if units == 'hPa':
-            pres = pres * 0.01
-        elif units == 'kPa':
-            pres = pres * 0.001
-        elif units != 'Pa':
-            raise ValueError('units must be kPa, hPa, or Pa.')
-
-        return pres
-
-
-    @staticmethod
-    def calc_eto(wrf_xr):
-        """
-
-        """
-        ## Assign variables
-        qv = wrf_xr['Q2']
-        pres = wrf_xr['PSFC']
-        gamma = (0.665*10**-3)*pres/1000
-        t2 = wrf_xr['T2'] - 273.15
-        G = wrf_xr['HFX'] * 0.0036
-        R_n = (wrf_xr['SWDOWN']*wrf_xr['ALBEDO'] + wrf_xr['GLW']) * 0.0036
-        # R_nl = wrf_xr['GLW'] * 0.0036
-        # alb = wrf_xr['ALBEDO']
-        u10 = wrf_xr['U10']
-        v10 = wrf_xr['V10']
-        ws2 = np.sqrt(u10**2 + v10**2)*4.87/(np.log(67.8*10 - 5.42))
-
-        # Humidity
-        e_mean = 0.6108*np.exp(17.27*t2/(t2+237.3))
-        qvs = 0.622*e_mean/(pres/100-(1-0.622)*e_mean)
-        rh = 100*qv/qvs
-        rh = xr.where(rh > 100, 100, rh)
-
-        # Vapor pressure
-        e_a = e_mean * rh/100
-        delta = 4098*(0.6108*np.exp(17.27*t2/(t2 + 237.3)))/((t2 + 237.3)**2)
-        # R_ns = (1 - alb)*R_s
-
-        # Calc ETo
-        ETo = (0.408*delta*(R_n - G) + gamma*37/(t2 + 273)*ws2*(e_mean - e_a))/(delta + gamma*(1 + 0.34*ws2))
-
-        return ETo
-
-
     def get_results(self, parameter_code=None, station_id_index=True):
         """
 
@@ -234,7 +103,7 @@ class WRF(object):
             map1 = self.mappings.loc[self.parameter_code].copy()
 
         if isinstance(map1['function'], str):
-            meth = getattr(self, map1['function'])
+            meth = getattr(self.vp, map1['function'])
             res1 = meth(self.data)
         else:
             res1 = self.data[map1['wrf_standard_name']]
@@ -258,7 +127,7 @@ class WRF(object):
         if parameter_code not in self.datasets:
             raise ValueError('parameter_code ' + parameter_code + ' is not available. Check the datasets dict for the available parameter codes.')
 
-        ## Remove prior parameter_code
+        ## Remove prior stored objects
         if hasattr(self, 'parameter_code'):
             delattr(self, 'parameter_code')
         if hasattr(self, 'param_dataset'):
@@ -267,6 +136,10 @@ class WRF(object):
             delattr(self, 'param_data')
         if hasattr(self, 'param_map'):
             delattr(self, 'param_map')
+        if hasattr(self, 'data_dict'):
+            delattr(self, 'data_dict')
+        if hasattr(self, 'run_date_dict'):
+            delattr(self, 'run_date_dict')
 
         ## Build the dataset
         datasets = self.datasets
@@ -283,7 +156,7 @@ class WRF(object):
         return ds1
 
 
-    def save_results(self, conn_config, bucket, public_url=None, run_date=None, threads=30):
+    def package_results(self, run_date=None):
         """
 
         """
@@ -304,94 +177,144 @@ class WRF(object):
         map1 = self.param_map.copy()
         data1['height']= map1['height']
         encoding = {ds1['parameter']: map1[['scale_factor', 'add_offset', 'dtype', '_FillValue']].dropna().to_dict()}
-        attrs = {ds1['parameter']: ds1.copy()}
+        # attrs = {ds1['parameter']: ds1.copy()}
+        # remote = {'bucket': bucket, 'connection_config': conn_config}
 
-        run_date_key = tu.make_run_date_key(run_date)
+        ds_id = ds1['dataset_id']
+        ds2 = ds1.copy()
+        ds2['properties'] = {'encoding': encoding}
 
-        s3 = tu.s3_connection(conn_config, threads)
+        max_run_date_key = tu.make_run_date_key(run_date)
+        run_date_dict = {ds_id: max_run_date_key}
 
-        ## Create the iterator for starmap
+        # run_date_dict = tu.process_run_date(3, [ds1], remote, run_date=run_date)
+        # max_run_date_key = max(list(run_date_dict.values()))
+
+        ## Create the data_dict
+        ds_id = ds1['dataset_id']
+        data_dict = {ds_id: []}
+
+        ## Prepare data
         stn_ids = data1.station_id.values.tolist()
-        inputs = iter((data1.sel(station_id=s).copy(), attrs, encoding, run_date_key, conn_config, bucket, s3, public_url) for s in stn_ids)
-        # inputs = iter((data1.sel(station_id=s).copy(), attrs, encoding, run_date_key, conn_config, bucket, s3, public_url) for s in stn_ids)
 
-        ## Run the threadpool
-        # output = ThreadPool(threads).imap_unordered(update_result, results)
-        with ThreadPool(threads) as pool:
-            # output = pool.starmap(self._save_results, inputs, chunksize=90)
-            output = pool.imap_unordered(self._save_results, inputs, chunksize=900)
-            pool.close()
-            pool.join()
+        for s in stn_ids[:1000]:
+        # for s in stn_ids:
+            print(s)
+            data2 = data1.sel(station_id=s).copy()
 
-        ## Final station and dataset agg
-        ds_new = tu.put_remote_dataset(s3, bucket, ds1)
-        ds_stations = tu.put_remote_agg_stations(s3, bucket, ds1['dataset_id'])
+            lat = float(data2['lat'].values)
+            lon = float(data2['lon'].values)
+            alt = round(float(data2['altitude'].values), 3)
+
+            geo1 = {"coordinates": [lon, lat], "type": "Point"}
+
+            stn_data = {'geometry': geo1, 'altitude': alt, 'station_id': s, 'virtual_station': True}
+
+            df1 = data2.drop(['lat', 'lon', 'altitude', 'station_id']).to_dataframe().reset_index()
+            df2 = df1.drop_duplicates('time', keep='first')
+
+            tu.prepare_results(data_dict, [ds2], stn_data, df2, max_run_date_key,  other_closed='left', discrete=False)
+
+        setattr(self, 'data_dict', data_dict)
+        setattr(self, 'run_date_dict', run_date_dict)
+        data1.close()
+        del data1
+        data2.close()
+        del data2
+        print('Finished packaging the data')
+
+
+    def save_results(self, processing_code, remote, public_url=None, threads=30):
+        """
+
+        """
+        if not hasattr(self, 'data_dict'):
+            raise ValueError('The package_results method must be run prior to saving data.')
+
+        tu.update_results_s3(processing_code, self.data_dict, self.run_date_dict, remote, threads=threads, public_url=public_url)
+
+        print('Finished saving data!')
+
+
+    def update_stations_datasets(self, processing_code, remote, threads=60):
+        """
+
+        """
+        s3 = tu.s3_connection(remote['connection_config'], threads)
+
+        ds = self.param_dataset.copy()
+
+        ds_new = tu.put_remote_dataset(s3, remote['bucket'], ds)
+        ds_stations = tu.put_remote_agg_stations(s3, remote['bucket'], ds['dataset_id'], threads)
 
         ### Aggregate all datasets for the bucket
-        ds_all = tu.put_remote_agg_datasets(s3, bucket)
+        ds_all = tu.put_remote_agg_datasets(s3, remote['bucket'], threads)
 
-        end1 = pd.Timestamp.now('utc').round('s')
-        print('End: ' + str(end1))
+        print('--Success!')
 
 
-    @staticmethod
-    def _save_results(val):
-        """
 
-        """
-        data, attrs, encoding, run_date, conn_config, bucket, s3, public_url = val
-        stn_id = str(data['station_id'].values)
 
-        print(stn_id)
 
-        lat = round(float(data['lat'].values), 6)
-        lon = round(float(data['lon'].values), 6)
-        alt = round(float(data['altitude'].values), 3)
 
-        geo1 = {"coordinates": [lon, lat], "type": "Point"}
+    # @staticmethod
+    # def _save_results(val):
+    #     """
 
-        stn_data = {'geometry': geo1, 'altitude': alt, 'station_id': stn_id, 'virtual_station': True}
+    #     """
+    #     data, attrs, encoding, run_date, conn_config, bucket, s3, public_url = val
+    #     stn_id = str(data['station_id'].values)
 
-        df1 = data.drop(['lat', 'lon', 'altitude', 'station_id']).to_dataframe().reset_index()
-        df2 = df1.drop_duplicates('time', keep='first').set_index(['time', 'height'])
-        parameter = df2.columns[0]
-        ds_id = attrs[parameter]['dataset_id']
+    #     print(stn_id)
 
-        new1 = tu.data_to_xarray(df2, stn_data, parameter, attrs, encoding, virtual_station=True)
+    #     lat = round(float(data['lat'].values), 6)
+    #     lon = round(float(data['lon'].values), 6)
+    #     alt = round(float(data['altitude'].values), 3)
 
-        up1 = tu.compare_datasets_from_s3(conn_config, bucket, new1, add_old=True, last_run_date_key=run_date, public_url=public_url)
+    #     geo1 = {"coordinates": [lon, lat], "type": "Point"}
 
-        ## Save results
-        if isinstance(up1, xr.Dataset) and (len(up1[parameter].time) > 0):
+    #     stn_data = {'geometry': geo1, 'altitude': alt, 'station_id': stn_id, 'virtual_station': True}
 
-            print('Save results')
-            key_dict = {'dataset_id': ds_id, 'station_id': stn_id, 'run_date': run_date}
+    #     df1 = data.drop(['lat', 'lon', 'altitude', 'station_id']).to_dataframe().reset_index()
+    #     df2 = df1.drop_duplicates('time', keep='first').set_index(['time', 'height'])
+    #     parameter = df2.columns[0]
+    #     ds_id = attrs[parameter]['dataset_id']
 
-            new_key = tu.key_patterns['results'].format(**key_dict)
+    #     new1 = tu.data_to_xarray(df2, stn_data, parameter, attrs, encoding, virtual_station=True)
 
-            cctx = zstd.ZstdCompressor(level=1)
-            c_obj = cctx.compress(up1.to_netcdf())
+    #     up1 = tu.compare_datasets_from_s3(conn_config, bucket, new1, add_old=True, last_run_date_key=run_date, public_url=public_url)
 
-            s3.put_object(Body=c_obj, Bucket=bucket, Key=new_key, ContentType='application/zstd', Metadata={'run_date': run_date})
+    #     ## Save results
+    #     if isinstance(up1, xr.Dataset) and (len(up1[parameter].time) > 0):
 
-            up1.close()
+    #         print('Save results')
+    #         key_dict = {'dataset_id': ds_id, 'station_id': stn_id, 'run_date': run_date}
 
-            ## Process stn data
-            print('Save station data')
+    #         new_key = tu.key_patterns['results'].format(**key_dict)
 
-            stn_m = tu.process_station_summ(ds_id, stn_id, conn_config, bucket, mod_date=run_date, public_url=public_url)
+    #         cctx = zstd.ZstdCompressor(level=1)
+    #         c_obj = cctx.compress(up1.to_netcdf())
 
-            stn4 = orjson.loads(stn_m.json(exclude_none=True))
-            up_stns = tu.put_remote_station(s3, bucket, stn4, run_date=run_date)
+    #         s3.put_object(Body=c_obj, Bucket=bucket, Key=new_key, ContentType='application/zstd', Metadata={'run_date': run_date})
 
-        else:
-            print('No new data to update')
+    #         up1.close()
 
-        ## Get rid of big objects
-        data.close()
-        new1.close()
-        new1 = None
-        up1 = None
+    #         ## Process stn data
+    #         print('Save station data')
+
+    #         stn_m = tu.process_station_summ(ds_id, stn_id, conn_config, bucket, mod_date=run_date, public_url=public_url)
+
+    #         stn4 = orjson.loads(stn_m.json(exclude_none=True))
+    #         up_stns = tu.put_remote_station(s3, bucket, stn4, run_date=run_date)
+
+    #     else:
+    #         print('No new data to update')
+
+    #     ## Get rid of big objects
+    #     data.close()
+    #     new1.close()
+    #     new1 = None
+    #     up1 = None
 
 
 
