@@ -978,14 +978,14 @@ def open_mf_wrf_dataset(paths, chunks=None,  compat='no_conflicts', lock=None,
         combined = xr.auto_combine(datasets, concat_dim='time', compat=compat)
     combined.attrs = datasets[0].attrs
 
-    combined.set_close(ds_closer)
+    # combined.set_close(ds_closer)
 
-    # try:
-    #     combined.set_close(ds_closer)
-    # except AttributeError:
-    #     from xarray.backends.api import _MultiFileCloser
-    #     mfc = _MultiFileCloser([ods._file_obj for ods in orig_datasets])
-    #     combined._file_obj = mfc
+    try:
+        combined.set_close(ds_closer)
+    except AttributeError:
+        from xarray.backends.api import _MultiFileCloser
+        mfc = _MultiFileCloser([ods._file_obj for ods in orig_datasets])
+        combined._file_obj = mfc
 
     # drop accumulated vars if needed (TODO: make this not hard coded)
     vns = ['PRCP', 'PRCP_C', 'PRCP_NC']
